@@ -1,6 +1,5 @@
 #! /bin/sh
-# Please have a TMP or TMPDIR environment variable if you don't trust /tmp,
-# or don't run this as root.
+# Please have a TMP or TMPDIR environment variable if you don't trust /tmp.
 
 export PATH=/usr/local/bin:/usr/local/sbin:$PATH
 
@@ -63,7 +62,6 @@ $dialog \
 'without-usernames' "Use only numerical UIDs/GIDs" off \
 'without-iplogging' "Never log remote IP addresses (confidentiality)" off \
 'without-humor' "Disable humor (enabled by default)" off \
-'without-longoptions' "Don't support long options (like --help)" off \
 'without-ascii' "Don't support 7-bits (ASCII) transfers" off \
 'without-banner' "Don't display the nice initial banner" off \
 'without-nonalnum' "Only allow minimal alpha-numeric characters" off \
@@ -87,11 +85,13 @@ $dialog \
 'with-quotas' "Support .ftpquota files" on \
 'with-welcomemsg' "welcome.msg files backward compatibility" off \
 'with-largefile' "Support for files > 2 Gb (slower)" off \
-'with-uploadscript' "Allow running pure-uploadscript after upload" on \
+'with-uploadscript' "Allow running scripts after upload (experimental)" on \
 'with-virtualhosts' "Allow a distinct content for each IP address" on \
 'with-virtualchroot' "Follow symlinks outside a chroot jail" off \
 'with-diraliases' "Support directory aliases" on \
 'with-peruserlimits' "Support per-user concurrency limits" on \
+'with-privsep' "Enable privilege separation" off \
+'with-tls' "Support SSL/TLS security layer (experimental)" off \
 2> $tmp
 
 cfgline='';
@@ -122,6 +122,7 @@ $dialog \
 'traditional-chinese' "Contributed by Hether Fygul" off \
 'simplified-chinese' "Contributed by Hether Fygul" off \
 'czech' "Contributed by Martin Sarfy" off \
+'turkish' "Contributed by Mehmet Cokcevik" off \
 2> $tmp
 
 z=$(cat $tmp)
