@@ -8,7 +8,7 @@
 # define __attribute__(a)
 #endif
 
-#ifdef SYSTEM_QUOTAS
+#if defined(SYSTEM_QUOTAS) || defined(WITH_PRIVSEP)
 # ifdef HAVE_SYS_FSUID_H
 #  undef HAVE_SYS_FSUID_H
 # endif
@@ -27,7 +27,7 @@
 #ifdef HAVE_LOCALE_H
 # include <locale.h>
 #endif
-#if HAVE_STRING_H
+#ifdef HAVE_STRING_H
 # if !STDC_HEADERS && HAVE_MEMORY_H
 #  include <memory.h>
 # endif
@@ -44,7 +44,7 @@
 #ifdef HAVE_UNISTD_H
 # include <unistd.h>
 #endif
-#if TIME_WITH_SYS_TIME
+#ifdef TIME_WITH_SYS_TIME
 # include <sys/time.h>
 # include <time.h>
 #else
@@ -116,14 +116,24 @@
 
 #include "ipv4stack.h"
 
-/* We can't have both implementations, so if both were
+/* We can't have more than one implementation, so if more than one were
  * found, the configure test failed - Use none */
-#if defined(DISABLE_SENDFILE) || (defined(SENDFILE_FREEBSD) && defined(SENDFILE_LINUX))
+#ifdef VIRTUOZZO
+# ifndef DISABLE_SENDFILE
+#  define DISABLE_SENDFILE 1
+# endif
+#endif
+#if defined(DISABLE_SENDFILE) || \
+  (defined(SENDFILE_FREEBSD) && (defined(SENDFILE_LINUX) || defined(SENDFILEV_SOLARIS) || defined(SENDFILE_HPUX))) || \
+  (defined(SENDFILE_LINUX) && (defined(SENDFILEV_SOLARIS) || defined(SENDFILE_HPUX))) || \
+  (defined(SENDFILEV_SOLARIS) && defined(SENDFILE_HPUX))
 # undef SENDFILE_FREEBSD
 # undef SENDFILE_LINUX
+# undef SENDFILE_HPUX
+# undef SENDFILEV_SOLARIS
 #endif
-#ifdef SENDFILE_FREEBSD
-/* FreeBSD sendfile() handles large files */
+#if defined(SENDFILE_FREEBSD) || defined(SENDFILEV_SOLARIS) || defined(SENDFILE_HPUX)
+/* FreeBSD, Solaris and HP-UX sendfile() handles large files */
 #elif defined(SENDFILE_LINUX)
 # ifdef WITH_LARGE_FILES
 #  undef SENDFILE_LINUX
@@ -136,7 +146,14 @@
 #ifdef HAVE_SYS_UIO_H
 # include <sys/uio.h>
 #endif
-#if defined(SENDFILE_LINUX) && defined(HAVE_SYS_SENDFILE_H)
+
+/*
+ * sendfile() is very kernel dependant. It's probable that you have platforms
+ * that require specific #include before sys/sendfile.h . So to enhance
+ * portability, we only include sys/sendfile.h on operating systems known
+ * to be supported
+ */
+#if (defined(SENDFILE_LINUX) || defined(SENDFILEV_SOLARIS)) && defined(HAVE_SYS_SENDFILE_H)
 # include <sys/sendfile.h>
 #endif
 
@@ -177,6 +194,10 @@
 # include <nlist.h>
 #endif
 
+#ifdef HAVE_SYS_PSTAT_H
+# include <sys/pstat.h>
+#endif
+
 #ifndef O_NOFOLLOW
 # define O_NOFOLLOW 0
 #endif
@@ -260,7 +281,10 @@ typedef struct AuthResult_ {
     unsigned int ratio_upload;
     unsigned int ratio_download;
     int ratio_ul_changed;
-    int ratio_dl_changed;    
+    int ratio_dl_changed;
+#endif
+#ifdef PER_USER_LIMITS
+    unsigned int per_user_max;
 #endif
 } AuthResult;
 
@@ -290,7 +314,7 @@ int safe_write(const int fd, const void *buf_, size_t count);
 void *aborttransfer(int);
 void parser(void);
 void stripctrl(char * const buf, size_t len);
-void dobanner(void);
+void dobanner(const int type);
 void douser(const char *name);
 void dopass(char *password);
 void docwd(const char *dir);
@@ -305,6 +329,10 @@ void dosize(const char *name);
 void doeprt(char *p);
 void doport(const char *arg);
 void doport2(struct sockaddr_storage a, unsigned int p);
+#ifndef MINIMAL
+void doesta(void);
+void doestp(void);
+#endif
 void dopasv(int);
 void dochmod(char *name, mode_t mode);
 void error(int n, const char *msg);
@@ -322,8 +350,10 @@ void mappedtov4(struct sockaddr_storage *ss);
 void disablesignals(void);
 #endif
 void getnames(void);
-void donlist(char *arg, int on_ctrlconn);
-int opendata(void);
+void donlist(char *arg, const int on_ctrlconn, const int opt_l_,
+             const int split_args);
+void opendata(void);
+void closedata(void);
 void addreply(const int code, const char * const line, ...)
 	__attribute__ ((format(printf, 2, 3)));
 void addreply_noformat(const int code, const char * const line);
@@ -344,6 +374,7 @@ const char *getgroup(const gid_t gid);
 const char *getname(const uid_t uid);
 unsigned int zrand(void);
 void simplify(char *subdir);
+int checkprintable(register const char *s);
 
 #ifdef HAVE_SYS_FSUID_H
 # define usleep2 usleep
@@ -363,6 +394,12 @@ extern int opt_a, opt_C, opt_d, opt_F, opt_l, opt_R;
 # define VHOST_PATH CONFDIR "/pure-ftpd"
 #endif
 
+#ifdef WITH_TLS
+# ifndef TLS_CERTIFICATE_PATH
+#  define TLS_CERTIFICATE_PATH CONFDIR "/ssl/private/pure-ftpd.pem"
+# endif
+#endif
+
 #define FAKE_SHELL "ftp"
 
 #ifndef PID_FILE
@@ -391,20 +428,38 @@ extern int opt_a, opt_C, opt_d, opt_F, opt_l, opt_R;
 
 #ifndef NON_ROOT_FTP
 # define DEFAULT_FTP_PORT_S "21"
+#elif defined(WIN32) || defined(_WIN32) || defined(__WIN32__) || defined(__CYGWIN__)
+# define DEFAULT_FTP_PORT_S "21"
 #else
 # define DEFAULT_FTP_PORT_S "2121"
 #endif
 
+/*
+ * Some operating systems (at least Solaris > 2.7 and FreeBSD) have strange
+ * troubles with reusing TCP ports, even when SO_REUSEADDR is enabled. 
+ * As a workaround, we try several unassigned privileged ports.
+ * The last way is to let the OS assign a port.
+ * For egress filtering, you can accept connections from ports <= 20
+ * to ports >= 1024.
+ */
+
+#define FTP_ACTIVE_SOURCE_PORTS { \
+    DEFAULT_FTP_DATA_PORT, 2U, 3U, 4U, 5U, 6U, 10U, 14U, 16U, 0U \
+}
+
 #ifndef MAXPATHLEN
 # ifdef PATH_MAX
 #  define MAXPATHLEN PATH_MAX
 # else
 #  define MAXPATHLEN 65536U
-Warning: neither PATH_MAX nor MAXPAHLEN were found.
+Warning: neither PATH_MAX nor MAXPATHLEN were found.
 Remove these lines if you really want to compile the server, but
 the server may be insecure if a wrong value is set here.    
 # endif
 #endif
+#if (MAXPATHLEN) >= (INT_MAX)
+Your platform has a very large MAXPATHLEN, we should not trust it.
+#endif
 
 #define DEFAULT_MAX_USERS 50
 #define DEFAULT_FTP_DATA_PORT 20 
@@ -416,6 +471,7 @@ the server may be insecure if a wrong value is set here.
 #define GLOB_TIMEOUT 17                   /* Max user time for a 'ls' to complete */
 #define MAX_CPU_TIME (17 * 60)           /* Max allowed CPU time per session */
 #define MAX_SESSION_XFER_IDLE (24 * 60 * 60)   /* Max duration of a transfer */
+#define MAX_USER_LENGTH 32U
 
 #ifdef LOG_FTP
 # define DEFAULT_FACILITY LOG_FTP
@@ -443,6 +499,8 @@ the server may be insecure if a wrong value is set here.
 #define MAX_UL_CHUNK_SIZE (CONF_TCP_SO_RCVBUF)
 #define MAX_THROTTLING_DELAY 42           /* Maximum throttling compensation */
 
+#define VHOST_PREFIX_MAX_LEN 64    
+    
 /*
  * Some users reported that something was wrong with TCP_CORK.
  * Better disable it for now, but we have to investigate.
@@ -590,6 +648,12 @@ the server may be insecure if a wrong value is set here.
 # define S_IXUGO (S_IXUSR | S_IXGRP | S_IXOTH)
 #endif
 
+#ifdef ACCEPT_UNICODE_CONTROL_CHARS
+# define ISCTRLCODE(X) ((X) == 0x7f || ((unsigned char) (X)) < 32U)
+#else
+# define ISCTRLCODE(X) ((X) == 0x7f || !(((unsigned char) (X)) & 0x60))
+#endif
+    
 #ifndef HAVE_MUNMAP
 # define munmap(A, B) (0)
 #endif
@@ -598,6 +662,25 @@ the server may be insecure if a wrong value is set here.
 # define gethostname(A, B) (-1)
 #endif
 
+#ifndef HAVE_SETEUID
+# ifdef HAVE_SETREUID
+#  define seteuid(X) setreuid(-1, (X))
+# elif defined(HAVE_SETRESUID)
+#  define seteuid(X) setresuid(-1, (X), -1)
+# else
+#  define seteuid(X) (-1)
+# endif
+#endif
+#ifndef HAVE_SETEGID
+# ifdef HAVE_SETREGID
+#  define setegid(X) setregid(-1, (X))
+# elif defined(HAVE_SETRESGID)
+#  define setegid(X) setresgid(-1, (X), -1)
+# else
+#  define setegid(X) (-1)
+# endif
+#endif
+    
 #define CRLF "\r\n"
 
 #ifdef WITH_DMALLOC
