@@ -15,13 +15,16 @@ static signed char auth_finalized;
 
 void pw_extauth_parse(const char * const file)
 {
-    if (file == NULL || *file == 0) {
+    size_t file_len;
+    
+    if (file == NULL || (file_len = strlen(file)) <= (size_t) 0U) {
         return;
     }
-    if ((saddr = malloc(sizeof(*saddr) + strlen(file) + 1U)) == NULL) {
+    if ((saddr = malloc(sizeof(*saddr) + file_len + 
+                        (size_t) 1U)) == NULL) {
         die_mem();
     }
-    strcpy(saddr->sun_path, file);   /* flawfinder: ignore - safe */
+    memcpy(saddr->sun_path, file, file_len + (size_t) 1U);
     saddr->sun_family = AF_UNIX;    
 }
 
@@ -128,6 +131,16 @@ static void callback_reply_ratio_download(const char *str, AuthResult * const re
 #endif
 }
 
+static void callback_reply_per_user_max(const char *str, AuthResult * const result)
+{
+#ifdef PER_USER_LIMITS
+    result->per_user_max = (unsigned int) strtoul(str, NULL, 10);
+#else
+    (void) str;
+    (void) result;
+#endif
+}
+
 static void callback_reply_end(const char *str, AuthResult * const result)
 {
     (void) str;
@@ -238,10 +251,12 @@ void pw_extauth_check(AuthResult * const result,
             scanned++;
         }
         linepnt = crpoint + 1;        
-    }        
-    if (result->uid <= (uid_t) 0 || result->gid <= (gid_t) 0 ||
-        result->dir == NULL || auth_finalized == 0) {
-        result->auth_ok = -1;
+    }
+    if (auth_finalized == 0 ||
+	(result->auth_ok == 1 && 
+	 (result->uid <= (uid_t) 0 || result->gid <= (gid_t) 0 || 
+	  result->dir == NULL))) {
+	result->auth_ok = -1;
     }
     bye:
     if (kindy != -1) {
