@@ -173,18 +173,17 @@
 #define MSG_WELCOME_TO "Benvenuto in"
 #define MSG_MAX_USERS "%lu utenti (il massimo) sono gi colleggati, spiacente"
 #define MSG_NB_USERS "Sei l'utente numero %d di %d consentiti"
-#define MSG_WELCOME_TIME "L'ora locale  %02d:%02d e il carico  %3.2f. Porta del server: %u."
+#define MSG_WELCOME_TIME "L'ora locale  %02d:%02d. Porta del server: %u."
 #define MSG_ANONYMOUS_FTP_ONLY "Qui  permesso solo FTP anonimo"
 #define MSG_RATIOS_EVERYONE "LE RATIOS SONO ABILITATE PER TUTTI:"
 #define MSG_RATIOS_ANONYMOUS "GLI UTENTI ANONIMI SONO SOGGETTI A UNA RATIO DI UL/DL  :"
 #define MSG_RATIOS_RULE "per downloadare %u Mb,  obbligatorio uploadare %u Mb di roba."
 #define MSG_INFO_IDLE_M "Sarai disconnesso dopo %lu minuti di inattivit."
 #define MSG_INFO_IDLE_S "Sarai disconnesso dopo  %lu secondi di inattivit."
-#define MSG_INFO_CPU_TIME "Logout - Tempo CPU consumato: %ld.%03ld secondi."
 #define MSG_CANT_READ_FILE "Spiacente, impossibile leggere [%s]"
 #define MSG_LS_TRUNCATED "Output troncato a %u corrispondenze"
 #define MSG_LS_SUCCESS "%u corrispondenze totali"
-#define MSG_LOGOUT "Logout"
+#define MSG_LOGOUT "Logout."
 #define MSG_AUTH_FAILED_LOG "Autenticazione falita per l'utente [%s]"
 #define MSG_ILLEGAL_UMASK "Umask non valida"
 #define MSG_STANDALONE_FAILED "Impossibile avviare il server in modalit standalone"
@@ -213,3 +212,9 @@
 #define MSG_ALIASES_UNKNOWN "Alias %s sconosciuto."
 #define MSG_ALIASES_BROKEN_FILE "File degli aliases danneggiato"
 #define MSG_ALIASES_LIST "Sono disponibili i seguenti aliases :"
+#define MSG_PERUSER_MAX "Non posso accettare pi di %lu connessioni dallo stesso utente"
+#define MSG_IPV6_OK "Anche le connessioni IPv6 sono benvenute su questo server"
+#define MSG_TLS_INFO "SSL/TLS: Abilitato %s con cifratura %s a %d bit"
+#define MSG_TLS_WEAK "SSL/TLS: Cifratura troppo debole"
+#define MSG_TLS_NEEDED "Spiacente, su questo server non sono ammesse sessioni in chiaro.\n" \
+    "Ricollegati usando i sistemi di sicurezza SSL/TLS."
