@@ -173,18 +173,17 @@
 #define MSG_WELCOME_TO "Velkommen til"
 #define MSG_MAX_USERS "%lu brukere (maks) er allerede logget p"
 #define MSG_NB_USERS "Du er bruker nummer %d av %d tillatte"
-#define MSG_WELCOME_TIME "Den lokale tiden er n %02d:%02d og belastningen er %3.2f. Server port: %u."
+#define MSG_WELCOME_TIME "Den lokale tiden er n %02d:%02d. Server port: %u."
 #define MSG_ANONYMOUS_FTP_ONLY "Kun anonym FTP er tillatt her"
 #define MSG_RATIOS_EVERYONE "FORHOLD ER SATT P FOR ALLE:"
 #define MSG_RATIOS_ANONYMOUS "ANONYME BRUKERE FR ET UL/DL FORHOLD:"
 #define MSG_RATIOS_RULE "for  laste ned %u Mb, s last opp %u Mb."
 #define MSG_INFO_IDLE_M "Du blir koplet fra etter %lu minutt innaktivitet."
 #define MSG_INFO_IDLE_S "Du blir koplet fra etter %lu sekund innaktivitet."
-#define MSG_INFO_CPU_TIME "Logg ut - CPU tid brukt: %ld.%03ld sekunder."
 #define MSG_CANT_READ_FILE "Beklager, kunne ikke lese [%s]"
 #define MSG_LS_TRUNCATED "Date er avrundet til %u treff"
 #define MSG_LS_SUCCESS "%u treff totalt"
-#define MSG_LOGOUT "Logg ut"
+#define MSG_LOGOUT "Logg ut."
 #define MSG_AUTH_FAILED_LOG "Godkjennelse misslyktes for [%s]"
 #define MSG_ILLEGAL_UMASK "Feil umask"
 #define MSG_STANDALONE_FAILED "Kan ikke starte en standalone server"
@@ -215,3 +214,7 @@
 #define MSG_ALIASES_LIST "Flgende alias er tilgjengelige :"
 #define MSG_PERUSER_MAX "Kan ikke tillate mer enn %lu tilkoplinger fra samme bruker"
 #define MSG_IPV6_OK "IPv6 forbindelser er ogs velkommen p denne serveren"
+#define MSG_TLS_INFO "SSL/TLS: Aktivert %s med %s, %d bits kryperingsnkkel"
+#define MSG_TLS_WEAK "SSL/TLS: Krypteringsnkkelen er for svak"
+#define MSG_TLS_NEEDED "Beklager, klartekst sessjoner er ikke tillatt p denne serveren.\n" \
+    "Vr vennlig  kople til p ny med SSL/TLS aktivert."
