@@ -61,21 +61,20 @@ static void dodaemonize(void)
             return;
         } else if (child != (pid_t) 0) {
             _exit(EXIT_SUCCESS);
-        } else {
-            if (setsid() == (pid_t) -1) {
+        } else if (setsid() == (pid_t) -1) {
                perror("Daemonization failed : setsid");
-            }
-            (void) chdir("/");
-            (void) closedesc_all(1);
         }
-    }    
+        (void) chdir("/");
+        (void) closedesc_all(1);
+    }
 }
 
 static int init(void)
 {
 #ifndef NON_ROOT_FTP
     if (geteuid() != (uid_t) 0) {
-        fprintf(stderr, "Sorry, but you have to be r00t to run this program\n");
+        fprintf(stderr, 
+		"Sorry, but you have to be r00t to run this program\n");
         return -1;
     }
 #endif
@@ -106,7 +105,8 @@ static int parseoptions(int argc, char *argv[])
 
     while ((fodder =
 #ifndef NO_GETOPT_LONG
-            getopt_long(argc, argv, GETOPT_OPTIONS, long_options, &option_index)
+            getopt_long(argc, argv, GETOPT_OPTIONS, long_options, 
+			&option_index)
 #else
             getopt(argc, argv, GETOPT_OPTIONS)
 #endif
@@ -117,7 +117,8 @@ static int parseoptions(int argc, char *argv[])
             break;
         }
         case 'g': {
-            char *nptr, *endptr;
+            const char *nptr;
+            char *endptr;
             
             nptr = optarg;
             endptr = NULL;
@@ -145,7 +146,8 @@ static int parseoptions(int argc, char *argv[])
             break;            
         }
         case 'u': {
-            char *nptr, *endptr;
+            const char *nptr;
+            char *endptr;
             
             nptr = optarg;
             endptr = NULL;
@@ -367,22 +369,24 @@ static void process(const int clientfd)
 #ifdef DO_AUTHD_TIMEOUT
     (void) alarm(AUTHD_SCRIPT_TIMEOUT);
 #endif
-    (void) execl(script, script, NULL);
+    (void) execl(script, script, (char *) NULL);
     
     _exit(EXIT_SUCCESS);
 }
 
 int listencnx(void)
 {
+    struct sockaddr_un *saddr;    
     int clientfd;
     int ret = -1;
-    struct sockaddr_un *saddr;
+    const size_t socketpath_len = strlen(socketpath);
         
-    if ((saddr = malloc(sizeof(*saddr) + strlen(socketpath) + 1U)) == NULL) {
+    if ((saddr = malloc(sizeof(*saddr) + socketpath_len + 
+                        (size_t) 1U)) == NULL) {
         perror("No more memory to listen to anything");
         goto bye;
     }
-    strcpy(saddr->sun_path, socketpath);   /* flawfinder: ignore - safe */
+    memcpy(saddr->sun_path, socketpath, socketpath_len + (size_t) 1U);
     saddr->sun_family = AF_UNIX;
     (void) unlink(socketpath);
     (void) umask(077);    
