@@ -2,12 +2,11 @@
 
 #include "ftpd.h"
 #include "quotas.h"
-#if !defined(NO_GETOPT_LONG) && !defined(HAVE_GETOPT_LONG)
-# include "gnu-getopt.h"
+
+#ifndef HAVE_GETOPT_LONG
+# include "bsd-getopt_long.h"
 #else
-# ifdef HAVE_GETOPT_H
-#  include <getopt.h>
-# endif
+# include <getopt.h>
 #endif
 
 #ifdef WITH_DMALLOC
@@ -356,7 +355,7 @@ int main(int argc, char *argv[])
     }
     if (isroot != 0) {
         if (doinitsupgroups(NULL, uid, gid) 
-#if defined(WIN32) || defined(_WIN32) || defined(__WIN32__) || defined (__CYGWIN__)            
+#if defined(WIN32) || defined(_WIN32) || defined(__WIN32__) || defined(__CYGWIN__)            
             & 0
 #endif
             != 0 ||
